@@ -18,7 +18,7 @@
  *        dir ...: directories to process
  *        lib ...: libraries to link
  *
- * Copyright 1994-1996 David Engel and Mitch D'Souza
+ * Copyright 1994-2000 David Engel and Mitch D'Souza
  *
  * This program may be used for any purpose as long as this
  * copyright notice is kept.
@@ -33,7 +33,8 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <a.out.h>
-#include <linux/elf.h>
+#include <link.h>
+#include <elf.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
@@ -149,9 +150,9 @@ char *is_shlib(const char *dir, const char *name, int *type,
     char *cp, *cp2;
     FILE *file;
     struct exec exec;
-    struct elfhdr *elf_hdr;
+    ElfW(Ehdr) *elf_hdr;
     struct stat statbuf;
-    char buff[1024];
+    char buff[4096];
 
     /* see if name is of the form libZ.so* */
     if ((strncmp(name, "lib", 3) == 0 || strncmp(name, "ld-", 3) == 0) && \
@@ -187,7 +188,7 @@ char *is_shlib(const char *dir, const char *name, int *type,
 		    warn("can't read header from %s, skipping", buff);
 		else if (N_MAGIC(exec) != ZMAGIC && N_MAGIC(exec) != QMAGIC)
 		{
-		    elf_hdr = (struct elfhdr *) &exec;
+		    elf_hdr = (ElfW(Ehdr) *) &exec;
 		    if (elf_hdr->e_ident[0] != 0x7f ||
 			strncmp(&elf_hdr->e_ident[1], "ELF",3) != 0)
 		    {
@@ -203,7 +204,8 @@ char *is_shlib(const char *dir, const char *name, int *type,
 			    expected_type=LIB_ANY;
 			}
 		        *type = LIB_ELF;
-		        good = readsoname(buff, file, expected_type, type);
+		        good = readsoname(buff, file, expected_type, type, 
+					  elf_hdr->e_ident[EI_CLASS]);
 			if (good == NULL || *islink)
 			{
 			    if (good != NULL)
@@ -252,8 +254,8 @@ char *is_shlib(const char *dir, const char *name, int *type,
 void link_shlib(const char *dir, const char *file, const char *so)
 {
     int change = 1;
-    char libname[1024];
-    char linkname[1024];
+    char libname[4096];
+    char linkname[4096];
     struct stat libstat;
     struct stat linkstat;
 
@@ -492,6 +494,34 @@ char *get_extpath(void)
     return res;
 }
 
+void usage(void)
+{
+    fprintf(stderr,
+	    "ldconfig - update shared library symlinks\n"
+	    "\n"
+	    "usage: ldconfig [-DvqnNX] [-f conf] [-C cache] [-r root] dir ...\n"
+	    "       ldconfig -l [-Dv] lib ...\n"
+	    "       ldconfig -p\n"
+	    "\t-D:\t\tdebug mode, don't update links\n"
+	    "\t-v:\t\tverbose mode, print things as we go\n"
+	    "\t-q:\t\tquiet mode, don't print warnings\n"
+	    "\t-n:\t\tdon't process standard directories\n"
+	    "\t-N:\t\tdon't update the library cache\n"
+	    "\t-X:\t\tdon't update the library links\n"
+	    "\t-l:\t\tlibrary mode, manually link libraries\n"
+	    "\t-p:\t\tprint the current library cache\n"
+	    "\t-f conf :\tuse conf instead of %s\n"
+	    "\t-C cache:\tuse cache instead of %s\n"
+	    "\t-r root :\tfirst, do a chroot to the indicated directory\n"
+	    "\tdir ... :\tdirectories to process\n"
+	    "\tlib ... :\tlibraries to link\n"
+	    "\n"
+	    "Copyright 1994-2000 David Engel and Mitch D'Souza\n",
+	    LDSO_CONF, LDSO_CACHE
+	    );
+    exit(EXIT_FATAL);
+}
+
 int main(int argc, char **argv)
 {
     int i, c;
@@ -547,10 +577,7 @@ int main(int argc, char **argv)
 	    chroot_dir = optarg;
 	    break;
 	default:
-	    fprintf(stderr, "usage: %s [-DvnNX] [-f conf] [-C cache] [-r root] dir ...\n", prog);
-	    fprintf(stderr, "       %s -l [-Dv] lib ...\n", prog);
-	    fprintf(stderr, "       %s -p\n", prog);
-	    exit(EXIT_FATAL);
+	    usage();
 	    break;
 
 	/* THE REST OF THESE ARE UNDOCUMENTED AND MAY BE REMOVED
@@ -702,7 +729,7 @@ void cache_write(void)
 {
     int cachefd;
     int stroffset = 0;
-    char tempfile[1024];
+    char tempfile[4096];
     liblist_t *cur_lib;
 
     if (!magic.nlibs)
@@ -781,17 +808,18 @@ void cache_print(void)
     for (fd = 0; fd < header->nlibs; fd++)
     {
 	printf("\t%s ", strs + libent[fd].sooffset);
-	switch (libent[fd].flags) 
+	switch (libent[fd].flags & ~LIB_ELF64) 
 	{
 	  case LIB_DLL:
 	    printf("(libc4)");
 	    break;
 	  case LIB_ELF:
-	    printf("(ELF)");
+	    printf("(ELF%s)", libent[fd].flags & LIB_ELF64 ? "/64" : "");
 	    break;
 	  case LIB_ELF_LIBC5:
 	  case LIB_ELF_LIBC6:
-	    printf("(libc%d)", libent[fd].flags + 3);
+	    printf("(libc%d%s)", (libent[fd].flags & ~LIB_ELF64) + 3,
+		   libent[fd].flags & LIB_ELF64 ? "/64" : "");
 	    break;
 	  default:
 	    printf("(unknown)");
